"use strict";
// noinspection JSUnusedGlobalSymbols
Object.defineProperty(exports, "__esModule", { value: true });
exports.RxMaybe = void 0;
const rxjs_1 = require("rxjs");
const maybe_1 = require("../maybe/maybe");
const operators_1 = require("rxjs/operators");
var RxMaybe;
(function (RxMaybe) {
    /**
     * Takes a default value and returns an RxJS mapping operator that emits extracted values from Maybes when they
     * are {@link Just} or the default value when they are {@link Nothing}.
     */
    function defaultTo(defaultValue) {
        return (0, operators_1.map)((0, maybe_1.fromMaybe)(defaultValue));
    }
    RxMaybe.defaultTo = defaultTo;
    /**
     * Returns an RxJS mapping operator that emits extracted values from Maybes when they are {@link Just} or
     * `null` when they are {@link Nothing}.
     */
    function extract() {
        return (0, operators_1.map)((m) => (0, maybe_1.isJust)(m) ? m.value : null);
    }
    RxMaybe.extract = extract;
    /**
     * Takes a Maybe value of an observable and returns an observable of maybe values.
     */
    function sequence(m) {
        return (0, maybe_1.isJust)(m) ? m.value.pipe((0, operators_1.map)(maybe_1.Just)) : (0, rxjs_1.of)(maybe_1.Nothing);
    }
    RxMaybe.sequence = sequence;
    /**
     * A {@link rxDistinctUntilChanged distinctUntilChanged} implementation that combines the native RxJS
     * operator with checks for Maybe values. Optionally takes a comparator function that decides whether
     * the values of two consecutive {@link Just} emissions are equivalent. Compares by identity by default.
     */
    function distinctUntilChanged(comp = (x, y) => x === y) {
        return (0, operators_1.distinctUntilChanged)((ma, mb) => {
            return ((0, maybe_1.isNothing)(ma) && (0, maybe_1.isNothing)(mb))
                || ((0, maybe_1.isJust)(ma) && (0, maybe_1.isJust)(mb) && comp(ma.value, mb.value));
        });
    }
    RxMaybe.distinctUntilChanged = distinctUntilChanged;
    /**
     * RxJS map operators that combine with Maybe methods
     */
    let map;
    (function (map) {
        /**
         * Takes a function and returns an RxJS mapping operator that fmaps the given
         * function over emitted Maybe values of the source observable.
         *
         * ─ signature ─
         *
         * ```
         * (T -> U) -> Observable (Maybe T) -> Observable (Maybe U)
         * ```
         */
        function fmap(f) {
            return (0, operators_1.map)((mx) => mx.fmap(f));
        }
        map.fmap = fmap;
        /**
         * Takes a Maybe value for a function and returns an RxJS mapping operator that applies it to
         * the maybe values of the source observable.
         *
         * ─ signature ─
         *
         * ```
         * Maybe (T -> U) -> Observable (Maybe T) -> Observable (Maybe U)
         * ```
         */
        function apply(mf) {
            return (0, operators_1.map)((mx) => mx.apply(mf));
        }
        map.apply = apply;
        /**
         * Takes a function and returns an RxJS mapping operator that binds it to
         * the maybe values of the source observable.
         *
         * ─ signature ─
         *
         * ```
         * (T -> Maybe U) -> Observable (Maybe T) -> Observable (Maybe U)
         * ```
         */
        function bind(f) {
            return (0, operators_1.map)((mx) => mx.bind(f));
        }
        map.bind = bind;
    })(map = RxMaybe.map || (RxMaybe.map = {}));
    /**
     * RxJS switchMap operators that combine with Maybe methods
     */
    let switchMap;
    (function (switchMap) {
        /**
         * Takes a function and returns an RxJS switchMapping operator that fmaps it over the values
         * of the source observable. The given function should itself return an observable.
         *
         * ─ signature ─
         *
         * ```
         * (T -> Observable U) -> Observable (Maybe T) -> Observable (Maybe U)
         * ```
         */
        function fmap(f) {
            return (0, operators_1.switchMap)((mx) => sequence(mx.fmap(f)));
        }
        switchMap.fmap = fmap;
        /**
         * Takes a function and returns an RxJS switchMapping operator that applies it to the maybe
         * values of the source observable. The given function should itself return an observable.
         *
         * ─ signature ─
         *
         * ```
         * Maybe (T -> Observable U) -> Observable (Maybe T) -> Observable (Maybe U)
         * ```
         */
        function apply(mf) {
            return (0, operators_1.switchMap)((mx) => sequence(mx.apply(mf)));
        }
        switchMap.apply = apply;
        /**
         * Takes a function and returns an RxJS switchMapping operator that binds it to the maybe
         * values of the source observable. The given function should itself return a maybe value
         * of an observable.
         *
         * ─ signature ─
         *
         * ```
         * (T -> Maybe (Observable U)) -> Observable (Maybe T) -> Observable (Maybe U)
         * ```
         */
        function bind(f) {
            return (0, operators_1.switchMap)((mx) => sequence(mx.bind(f)));
        }
        switchMap.bind = bind;
        /**
         * Takes a function and returns an RxJS switchMapping operator that applies it to the
         * maybe values of the source observable if they are {@link Just}. The operator emits
         * Nothing when the input value is Nothing. The given function should itself return
         * an observable of maybe values.
         *
         * ─ signature ─
         *
         * ```
         * (T -> Observable (Maybe U)) -> Observable (Maybe T) -> Observable (Maybe U)
         * ```
         */
        function id(f) {
            return (0, operators_1.switchMap)((mx) => (0, maybe_1.isJust)(mx) ? f(mx.value) : (0, rxjs_1.of)(maybe_1.Nothing));
        }
        switchMap.id = id;
    })(switchMap = RxMaybe.switchMap || (RxMaybe.switchMap = {}));
})(RxMaybe || (exports.RxMaybe = RxMaybe = {}));
