import { isFunction } from '../assertions/assertions';
import { show } from '../utils/utils';
export const JUST_TAG = Symbol('TAG: Maybe.Just');
export const NOTHING_TAG = Symbol('TAG: Maybe.Nothing');
/**
 * Data instance for the Nothing {@link Maybe} member.
 */
export const Nothing = Object.freeze({
    TAG: NOTHING_TAG,
    fmap: () => Nothing,
    apply: () => Nothing,
    bind: () => Nothing,
    toString: () => `Nothing`,
});
/**
 * Data constructor for the Just {@link Maybe} member.
 */
export const Just = (x) => Object.freeze({
    TAG: JUST_TAG,
    value: x,
    fmap: (f) => Just(f(x)),
    apply: (mf) => expectMaybe(mf).fmap((f) => f(x)),
    bind: (f) => expectMaybe(f(x)),
    toString: () => `Just (${show(x)})`,
});
/**
 * Takes a default value, a function, and a Maybe value. If the Maybe value is Nothing, the function returns the
 * default value. Otherwise, it applies the function to the value inside the Just and returns the result.
 */
export const maybe = (x) => (f) => (m) => isJust(m) ? f(m.value) : x;
/**
 * Takes a default value and returns a new function that takes a Maybe. If the Maybe is {@link Nothing}
 * it returns the default value. If the Maybe is {@link Just} it returns the contained value.
 */
export const fromMaybe = (x) => (m) => isJust(m) ? m.value : x;
/**
 * Tells if the given value is a {@link Just} instance of Maybe.
 */
export function isJust(m) {
    try {
        return m.TAG === JUST_TAG
            && m.hasOwnProperty('value')
            && isFunction(m.fmap)
            && isFunction(m.apply)
            && isFunction(m.bind);
    }
    catch (error) {
        return false;
    }
}
/**
 * Tells if the given value is a {@link Nothing} instance of Maybe.
 */
export function isNothing(m) {
    try {
        return m.TAG === NOTHING_TAG
            && isFunction(m.fmap)
            && isFunction(m.apply)
            && isFunction(m.bind);
    }
    catch (error) {
        return false;
    }
}
function expectMaybe(x) {
    if (!isJust(x) && !isNothing(x)) {
        throw new Error('Expected maybe value but got ' + String(x));
    }
    return x;
}
