// noinspection JSUnusedGlobalSymbols
import { of } from 'rxjs';
import { fromMaybe, isJust, isNothing, Just, Nothing } from '../maybe/maybe';
import { map as rxMap, distinctUntilChanged as rxDistinctUntilChanged, switchMap as rxSwitchMap } from 'rxjs/operators';
export var RxMaybe;
(function (RxMaybe) {
    /**
     * Takes a default value and returns an RxJS mapping operator that emits extracted values from Maybes when they
     * are {@link Just} or the default value when they are {@link Nothing}.
     */
    function defaultTo(defaultValue) {
        return rxMap(fromMaybe(defaultValue));
    }
    RxMaybe.defaultTo = defaultTo;
    /**
     * Returns an RxJS mapping operator that emits extracted values from Maybes when they are {@link Just} or
     * `null` when they are {@link Nothing}.
     */
    function extract() {
        return rxMap((m) => isJust(m) ? m.value : null);
    }
    RxMaybe.extract = extract;
    /**
     * Takes a Maybe value of an observable and returns an observable of maybe values.
     */
    function sequence(m) {
        return isJust(m) ? m.value.pipe(rxMap(Just)) : of(Nothing);
    }
    RxMaybe.sequence = sequence;
    /**
     * A {@link rxDistinctUntilChanged distinctUntilChanged} implementation that combines the native RxJS
     * operator with checks for Maybe values. Optionally takes a comparator function that decides whether
     * the values of two consecutive {@link Just} emissions are equivalent. Compares by identity by default.
     */
    function distinctUntilChanged(comp = (x, y) => x === y) {
        return rxDistinctUntilChanged((ma, mb) => {
            return (isNothing(ma) && isNothing(mb))
                || (isJust(ma) && isJust(mb) && comp(ma.value, mb.value));
        });
    }
    RxMaybe.distinctUntilChanged = distinctUntilChanged;
    /**
     * RxJS map operators that combine with Maybe methods
     */
    let map;
    (function (map) {
        /**
         * Takes a function and returns an RxJS mapping operator that fmaps the given
         * function over emitted Maybe values of the source observable.
         *
         * ─ signature ─
         *
         * ```
         * (T -> U) -> Observable (Maybe T) -> Observable (Maybe U)
         * ```
         */
        function fmap(f) {
            return rxMap((mx) => mx.fmap(f));
        }
        map.fmap = fmap;
        /**
         * Takes a Maybe value for a function and returns an RxJS mapping operator that applies it to
         * the maybe values of the source observable.
         *
         * ─ signature ─
         *
         * ```
         * Maybe (T -> U) -> Observable (Maybe T) -> Observable (Maybe U)
         * ```
         */
        function apply(mf) {
            return rxMap((mx) => mx.apply(mf));
        }
        map.apply = apply;
        /**
         * Takes a function and returns an RxJS mapping operator that binds it to
         * the maybe values of the source observable.
         *
         * ─ signature ─
         *
         * ```
         * (T -> Maybe U) -> Observable (Maybe T) -> Observable (Maybe U)
         * ```
         */
        function bind(f) {
            return rxMap((mx) => mx.bind(f));
        }
        map.bind = bind;
    })(map = RxMaybe.map || (RxMaybe.map = {}));
    /**
     * RxJS switchMap operators that combine with Maybe methods
     */
    let switchMap;
    (function (switchMap) {
        /**
         * Takes a function and returns an RxJS switchMapping operator that fmaps it over the values
         * of the source observable. The given function should itself return an observable.
         *
         * ─ signature ─
         *
         * ```
         * (T -> Observable U) -> Observable (Maybe T) -> Observable (Maybe U)
         * ```
         */
        function fmap(f) {
            return rxSwitchMap((mx) => sequence(mx.fmap(f)));
        }
        switchMap.fmap = fmap;
        /**
         * Takes a function and returns an RxJS switchMapping operator that applies it to the maybe
         * values of the source observable. The given function should itself return an observable.
         *
         * ─ signature ─
         *
         * ```
         * Maybe (T -> Observable U) -> Observable (Maybe T) -> Observable (Maybe U)
         * ```
         */
        function apply(mf) {
            return rxSwitchMap((mx) => sequence(mx.apply(mf)));
        }
        switchMap.apply = apply;
        /**
         * Takes a function and returns an RxJS switchMapping operator that binds it to the maybe
         * values of the source observable. The given function should itself return a maybe value
         * of an observable.
         *
         * ─ signature ─
         *
         * ```
         * (T -> Maybe (Observable U)) -> Observable (Maybe T) -> Observable (Maybe U)
         * ```
         */
        function bind(f) {
            return rxSwitchMap((mx) => sequence(mx.bind(f)));
        }
        switchMap.bind = bind;
        /**
         * Takes a function and returns an RxJS switchMapping operator that applies it to the
         * maybe values of the source observable if they are {@link Just}. The operator emits
         * Nothing when the input value is Nothing. The given function should itself return
         * an observable of maybe values.
         *
         * ─ signature ─
         *
         * ```
         * (T -> Observable (Maybe U)) -> Observable (Maybe T) -> Observable (Maybe U)
         * ```
         */
        function id(f) {
            return rxSwitchMap((mx) => isJust(mx) ? f(mx.value) : of(Nothing));
        }
        switchMap.id = id;
    })(switchMap = RxMaybe.switchMap || (RxMaybe.switchMap = {}));
})(RxMaybe || (RxMaybe = {}));
