import { Observable, OperatorFunction } from 'rxjs';
import { Maybe } from '../maybe/maybe';
import { Fn, Fn2 } from '../types';
export declare namespace RxMaybe {
    type ObservableMaybe<T> = Observable<Maybe<T>>;
    type MaybeOperator<T, U> = OperatorFunction<Maybe<T>, Maybe<U>>;
    /**
     * Takes a default value and returns an RxJS mapping operator that emits extracted values from Maybes when they
     * are {@link Just} or the default value when they are {@link Nothing}.
     */
    function defaultTo<T>(defaultValue: T): OperatorFunction<Maybe<T>, T>;
    /**
     * Returns an RxJS mapping operator that emits extracted values from Maybes when they are {@link Just} or
     * `null` when they are {@link Nothing}.
     */
    function extract<T>(): OperatorFunction<Maybe<T>, T | null>;
    /**
     * Takes a Maybe value of an observable and returns an observable of maybe values.
     */
    function sequence<T>(m: Maybe<Observable<T>>): ObservableMaybe<T>;
    /**
     * A {@link rxDistinctUntilChanged distinctUntilChanged} implementation that combines the native RxJS
     * operator with checks for Maybe values. Optionally takes a comparator function that decides whether
     * the values of two consecutive {@link Just} emissions are equivalent. Compares by identity by default.
     */
    function distinctUntilChanged<T>(comp?: Fn2<T, T, boolean>): MaybeOperator<T, T>;
    /**
     * RxJS map operators that combine with Maybe methods
     */
    namespace map {
        /**
         * Takes a function and returns an RxJS mapping operator that fmaps the given
         * function over emitted Maybe values of the source observable.
         *
         * ─ signature ─
         *
         * ```
         * (T -> U) -> Observable (Maybe T) -> Observable (Maybe U)
         * ```
         */
        function fmap<T, U>(f: Fn<T, U>): MaybeOperator<T, U>;
        /**
         * Takes a Maybe value for a function and returns an RxJS mapping operator that applies it to
         * the maybe values of the source observable.
         *
         * ─ signature ─
         *
         * ```
         * Maybe (T -> U) -> Observable (Maybe T) -> Observable (Maybe U)
         * ```
         */
        function apply<T, U>(mf: Maybe<Fn<T, U>>): MaybeOperator<T, U>;
        /**
         * Takes a function and returns an RxJS mapping operator that binds it to
         * the maybe values of the source observable.
         *
         * ─ signature ─
         *
         * ```
         * (T -> Maybe U) -> Observable (Maybe T) -> Observable (Maybe U)
         * ```
         */
        function bind<T, U>(f: Fn<T, Maybe<U>>): MaybeOperator<T, U>;
    }
    /**
     * RxJS switchMap operators that combine with Maybe methods
     */
    namespace switchMap {
        /**
         * Takes a function and returns an RxJS switchMapping operator that fmaps it over the values
         * of the source observable. The given function should itself return an observable.
         *
         * ─ signature ─
         *
         * ```
         * (T -> Observable U) -> Observable (Maybe T) -> Observable (Maybe U)
         * ```
         */
        function fmap<T, U>(f: Fn<T, Observable<U>>): MaybeOperator<T, U>;
        /**
         * Takes a function and returns an RxJS switchMapping operator that applies it to the maybe
         * values of the source observable. The given function should itself return an observable.
         *
         * ─ signature ─
         *
         * ```
         * Maybe (T -> Observable U) -> Observable (Maybe T) -> Observable (Maybe U)
         * ```
         */
        function apply<T, U>(mf: Maybe<Fn<T, Observable<U>>>): MaybeOperator<T, U>;
        /**
         * Takes a function and returns an RxJS switchMapping operator that binds it to the maybe
         * values of the source observable. The given function should itself return a maybe value
         * of an observable.
         *
         * ─ signature ─
         *
         * ```
         * (T -> Maybe (Observable U)) -> Observable (Maybe T) -> Observable (Maybe U)
         * ```
         */
        function bind<T, U>(f: Fn<T, Maybe<Observable<U>>>): MaybeOperator<T, U>;
        /**
         * Takes a function and returns an RxJS switchMapping operator that applies it to the
         * maybe values of the source observable if they are {@link Just}. The operator emits
         * Nothing when the input value is Nothing. The given function should itself return
         * an observable of maybe values.
         *
         * ─ signature ─
         *
         * ```
         * (T -> Observable (Maybe U)) -> Observable (Maybe T) -> Observable (Maybe U)
         * ```
         */
        function id<T, U>(f: Fn<T, Observable<Maybe<U>>>): MaybeOperator<T, U>;
    }
}
